@@ -288,7 +288,8 @@ class DbOne(Speaker):
 
         except QueryError, e:
             DbOne._rollback()
-            Speaker.log_raise('Rollbacked queries: %s.', ERR_COL(e), QueryError)
+            Speaker.log_raise('%s queries: %s.', 
+                    ERR_COL('Rollbacked'), ERR_COL(e), QueryError)
 
 
     @staticmethod
@@ -380,9 +381,11 @@ class DbFly(Speaker):
 
         self.session_name= name or self._default_session
         self._import_iface()
-        self._connect()
+        #self._connect()
+        self._queries_n= 0
 
     def _import_iface(self):
+
         iface= self._iface_instance= _DbInfo.iface_builder()
 
         for attr in self._iface_attrs:
@@ -413,12 +416,14 @@ class DbFly(Speaker):
         self.log_debug("Rollbacking queries for Session %s", 
                 self.session_name)
         self._rollback()
+        self._queries_n= 0
 
     def commit(self):
         try:
+            self.log_info("Syncing (%s) queries for Session %s.", 
+                    OKI_COL(self._queries_n), OKI_COL(self.session_name))
             self._commit()
-            self.log_debug("Queries Committed for Session %s.", 
-                    OKI_COL(self.session_name))
+            self._queries_n= 0
             return 0
         except Exception, ss: 
             return ss
@@ -465,7 +470,9 @@ class DbFly(Speaker):
             self.log_info("%s: QSQL- %s, Filter= %s" , INF_COL(self.session_name), s, d)
 
         try:
-            return execattr(s, d)
+            res= execattr(s, d)
+            self._queries_n+= 1
+            return res
 
         except BrokenConnection, e:
 
@@ -485,7 +492,8 @@ class DbFly(Speaker):
 
         except QueryError, e:
             self._rollback()
-            self.log_raise('Rollbacked queries: %s.', ERR_COL(e), QueryError)
+            self.log_raise('%s queries: %s.', ERR_COL('Rollbacked'), 
+                    ERR_COL(e), QueryError)
 
 
     def _get_query(self, s, filter= None):
@@ -839,7 +847,7 @@ def init_abdbda(conf, forced= {}):
 
     if not pool or db_one:
         DbOne._import_iface()
-        DbOne._connect()
+        #DbOne._connect()
         DbOne._db_initialized= True
 
     if not pool:
