@@ -52,6 +52,9 @@ from abdbda import _J, _A
 ##
 #
 
+def _do_nothing(*args):
+    pass
+
 def init_mother(cfile, fnaming= None):
     """ init_mother(cfile [fnaming= None]) -> None
 
@@ -242,7 +245,6 @@ class _DbMap(DbOne):
     @staticmethod
     def _searchRelation(tbls):
 
-        from speaker import Speaker
         fz= frozenset(tbls)
 
         trd= _DbMap._map_rels
@@ -253,6 +255,7 @@ class _DbMap(DbOne):
         except:
             pass
 
+        from speaker import Speaker
         Speaker.log_insane(\
             "Warning: direct relation for tables %s "
             "is not defined: searching subset...",
@@ -262,7 +265,7 @@ class _DbMap(DbOne):
 
         if len(rels)<>1:
             Speaker.log_int_raise(
-                "In _searchRelation: can not get the "  
+                "In _searchRelation: can not choose the "  
                 "relation table. There are %s possible "     
                 "choices: %s .", ERR_COL(len(rels)), 
                 ERR_COL(_J(rels)))
@@ -271,11 +274,160 @@ class _DbMap(DbOne):
 
     @staticmethod
     def _isChildOf(c, f):
-        return c in _DbMap._map_children[f]
+        try:
+            return c in _DbMap._map_children[f]
+        except:
+            return False
 
-def _do_nothing(*args):
-    pass
+    @staticmethod
+    def _sqlFreeJoin(otbl, rtbl, jtbl, ord, jrd):
 
+        s= "JOIN %s ON %s" % (rtbl, 
+           _A(
+            [" %s.%s = %s.%s " % (otbl, k, rtbl, v)
+                for k, v in ord.iteritems()]))
+
+        r= "JOIN %s ON %s" % (jtbl, 
+           _A(
+            [" %s.%s = %s.%s " % (jtbl, k, rtbl, v)
+                for k, v in jrd.iteritems()]))
+
+        return s + r
+
+    @staticmethod
+    def _sqlJoin(j_tbl, r_tbl, j_key, r_key):
+        """ _sqlJoin(j_tbl, r_tbl, j_key, r_key) --> str
+
+        Returns the join query.
+        """
+
+        s= "JOIN %s ON %s" % (r_tbl, 
+           _A(
+            [" %s.%s = %s.%s " % (r_tbl, v, j_tbl, k)
+                for k, v in j_key.iteritems()]))
+
+        s+=" WHERE "
+        s+=_A(
+            ["%s.%s = %s " % (r_tbl, v, DbMother._mo_arg_format % k)
+                for k, v in r_key.iteritems()])
+
+        return s
+
+    def _sqlJoinParent(self, builder, j_builder, outer=False):
+        """ _sqlJoinParent(self, builder, j_builder [, outer= False]) --> (string, string)
+
+        Returns a filter and the table where filter has to be applied.
+        """
+
+        tbl= builder.table_name
+        j_tbl= j_builder.table_name
+
+        if not self._isChildOf(j_tbl, tbl):
+
+            try:
+                j_tbl= self._searchRelation([tbl, j_tbl])
+            except:
+                self.log_int_raise('_sqlJoinParent: no way to join |%s| with |%s|.',
+                                    ERR_COL(j_tbl), ERR_COL(tbl))
+
+        m_d= self.getChildDeps(j_tbl, builder)
+
+        if outer:
+            s=" LEFT OUTER JOIN %s on " % j_tbl
+        else:
+            s=" JOIN %s on " % j_tbl
+
+        s+=_A([" %(tbl)s.%(k)s = %(j_tbl)s.%(v)s " % locals() 
+                                for k, v in m_d.iteritems()])
+
+        return s, j_tbl
+
+    def _getRelationObject(self, builders):
+        """ _getRelationObject(builders): --> MotherFly
+
+        builders is a list of table names.
+        """
+        
+        rel_tbl_name= self._searchRelation(builders+ [self.table_name])
+
+        # Return a builder with MotherOnTheFly.
+        rel_obj= getMotherObj(rel_tbl_name, session= self.session)
+        return rel_obj
+
+    def getChildDeps(self, tbl_name, father= None):
+        """ getChildDeps(self, tbl_name [, father= None]) --> dict
+
+        Wich fields in TABLE tbl_name will be inherited from father?
+        Returned dict is the mapping_dict.
+        If father is None, self is assumed.
+        """
+
+        father= father or self
+        tbl= father.table_name
+
+        # Load father children.....
+        d= self._table_children(tbl)
+
+        # Get the dependencies for child tbl_name
+        try:
+            dd= d[tbl_name].copy()
+        except:
+            self.log_int_raise(\
+                "Child %s not specified for table %s.",\
+                    ERR_COL(tbl_name),ERR_COL(tbl))
+
+        return dd
+
+    def _getDepDict(self, mapping_dict, father= None):
+        """_getDepDict(mapping_dict [,father= None]): ---> dict
+
+        Given the dependencies mapping_dict, a new dict is created,
+        filled and returned with father values.
+        If father is None, self is assumed.
+	    Note: mapping_dict is {key_child: key_father}
+        """
+
+        father= father or self
+
+        self.log_insane( 
+                "Exporting values %s ",
+                str(mapping_dict))
+                        
+        fkeys= set(mapping_dict.keys())
+
+        try:
+            d= father.getFields(fkeys)
+        except Exception, s:
+            self.log_int_raise("Cannot inherit needed values from father: %s.",
+                    ERR_COL(list(fkeys)))
+            
+        new_d={}
+        for mapkey, mapped_key in mapping_dict.iteritems():
+            #mapped_field= mapped_key
+            new_d[mapped_key]= d[mapkey]
+            #new_d[mapkey]= d[mapkey]
+
+        return new_d
+
+    def _fillChildDict(self, child_obj, father= None):
+        """_fillChildDict(child_obj [,father= None]) --> None
+        Given a child object, his store is updatetd with
+        father dependencies. If father is None, father= self.
+        """
+        
+        father= father or self
+        ct= child_obj.table_name
+        
+        # Ge dependencies ...
+        d= self.getChildDeps(ct,father)
+        # Create new dict ...
+        new_d= self._getDepDict(d,father)
+        # Init child with new_dict
+        child_obj.setFields(new_d, safe_mode= False)
+
+        self.log_insane( 
+                "Child object(%s) updated with values %s.",\
+                OKI_COL(ct),OKI_COL(new_d))
 #
 ##
 ### The Mother Class
@@ -1023,6 +1175,7 @@ class DbMother(_DbMap):
 
 
 class MotherFly(DbMother):
+
     def __init__(self, tbl):
         self.table_name= tbl
 
@@ -1077,7 +1230,6 @@ def getMotherObj(tbl, d={}, flag= MO_NOA, session= None):
 #
 #   initChildManager(children_list)
 #   initRelationManager(children_list)
-#   initListManager(children_list)
 #
 ###
 ##
@@ -1094,137 +1246,6 @@ class MotherManager:
 ##
 #
 
-    def _sqlJoin(self, j_tbl, r_tbl, j_key, r_key):
-        """ _sqlJoin(self, j_tbl, r_tbl, j_key, r_key) --> str
-
-        Returns the join query.
-        """
-
-        s= "JOIN %s ON %s" % (r_tbl, 
-           _A(
-            [" %s.%s = %s.%s " % (r_tbl,v,j_tbl,k)
-                for k,v in j_key.iteritems()]))
-
-        s+=" WHERE "
-        s+=_A(
-            ["%s.%s = %%(%s)s " % (r_tbl,v,k)
-                for k,v in r_key.iteritems()])
-
-        return s
-
-    def _sqlJoinParent(self, builder, j_builder, outer=False):
-        """ _sqlJoinParent(self, builder, j_builder [, outer= False]) --> (string, string)
-
-        Returns a filter and the table where filter has to be applied.
-        """
-
-        tbl= builder.table_name
-        j_tbl= j_builder.table_name
-
-        if not self._isChildOf(j_tbl, tbl):
-
-            try:
-                j_tbl= self._searchRelation([tbl, j_tbl])
-            except:
-                self.log_int_raise('_sqlJoinParent: no way to join |%s| with |%s|.',
-                                    ERR_COL(j_tbl), ERR_COL(tbl))
-
-        m_d= self.getChildDeps(j_tbl, builder)
-
-        if outer:
-            s=" LEFT OUTER JOIN %s on " % j_tbl
-        else:
-            s=" JOIN %s on " % j_tbl
-
-        s+=_A([" %(tbl)s.%(k)s = %(j_tbl)s.%(v)s " % locals() for k,v in m_d.iteritems()])
-
-        return s, j_tbl
-
-    def _getRelationObject(self, builders):
-        """ _getRelationObject(builders): --> MotherFly
-
-        builders is a list of table names.
-        """
-        
-        rel_tbl_name= self._searchRelation(builders+ [self.table_name])
-
-        # Return a builder with MotherOnTheFly.
-        rel_obj= getMotherObj(rel_tbl_name, session= self.session)
-        return rel_obj
-
-    def getChildDeps(self, tbl_name, father= None):
-        """ getChildDeps(self, tbl_name [, father= None]) --> dict
-
-        Wich fields in TABLE tbl_name will be inherited from father?
-        Returned dict is the mapping_dict.
-        If father is None, self is assumed.
-        """
-
-        father=father or self
-        tbl= father.table_name
-
-        # Load father children.....
-        d= self._table_children(tbl)
-
-        # Get the dependencies for child tbl_name
-        try:
-            dd= d[tbl_name].copy()
-        except:
-            self.log_int_raise(\
-                "Child %s not specified for table %s.",\
-                    ERR_COL(tbl_name),ERR_COL(tbl))
-
-        return dd
-
-    def _getDepDict(self, mapping_dict, father= None):
-        """_getDepDict(mapping_dict [,father= None]): ---> dict
-
-        Given the dependencies mapping_dict, a new dict is created,
-        filled and returned with father values.
-        If father is None, self is assumed.
-        """
-
-        father= father or self
-        new_d={}
-
-        self.log_insane( 
-                "Exporting values %s ",
-                str(mapping_dict))
-                        
-        fkeys= set(mapping_dict.keys())
-
-        try:
-            d= father.getFields(fkeys)
-        except Exception, s:
-            self.log_int_raise("Cannot inherit needed values from father: %s.",
-                    ERR_COL(list(fkeys)))
-            
-        for mapkey in mapping_dict:
-            mapped_field= mapping_dict[mapkey]
-            new_d[mapped_field]= d[mapkey]
-
-        return new_d
-
-    def _fillChildDict(self, child_obj, father= None):
-        """_fillChildDict(child_obj [,father= None]) --> None
-        Given a child object, his store is updatetd with
-        father dependencies. If father is None, father= self.
-        """
-        
-        father= father or self
-        ct= child_obj.table_name
-        
-        # Ge dependencies ...
-        d= self.getChildDeps(ct,father)
-        # Create new dict ...
-        new_d= self._getDepDict(d,father)
-        # Init child with new_dict
-        child_obj.setFields(new_d, safe_mode= False)
-
-        self.log_insane( 
-                "Child object(%s) updated with values %s.",\
-                OKI_COL(ct),OKI_COL(new_d))
-
     def _workChild(self, child_builder, child_dict, flag):
     
         # Build obj...
@@ -2059,3 +2080,222 @@ class MotherBox(DbOne):
             "Obsolete Method MotherBox.getFields(). Use getRecords() instead! "
             "This method will be removed from Mother."))
         return self.getRecords(flag_obj)
+
+#
+##
+### The Modern MotherFusion
+##
+#
+
+class MotherFusion(_DbMap):
+
+    def __init__(self, builderA, builderB, filter= None, fields= None,
+                    order= None, session= None, distinct= False, 
+                    rtbl= None, params= False, jfilter= None):
+
+        self.direct= None
+
+        builderA, builderB, fields= self.swap(builderA, builderB, fields)
+
+        if session: session._export_iface(self)
+        self.session= session
+
+        if filter:
+            if not isinstance(filter, MoFilter):
+                filter= MoFilter(filter)
+        else:
+            filter= MoFilter()
+        self.filter= filter
+
+        self.jfilter= jfilter
+
+
+        if order:
+            self.filter.add_post('ORDER BY %s' % _J(order))
+
+        self.distinct= distinct
+        self._store= []
+        self.rtbl= None
+        self.params= params
+
+        self.builderA= builderA
+        self.builderB= builderB
+        self.tblA= ta= builderA.table_name
+        self.tblB= tb= builderB.table_name
+
+        if fields:
+            self.fields= fields
+        else:
+            self.fields= (self._table_fields(ta), self._table_fields(tb))
+
+        if self.direct:
+            self.directJoin()
+        else:
+            self.rtbl= rtbl
+            self.joinBuilders()
+
+    def swap(self, a, b, fields):
+
+        ta= a.table_name
+        tb= b.table_name
+
+        if self._isChildOf(ta, tb):
+
+            self.direct= True
+
+            if fields and isinstance(fields, tuple):
+                fields= (fields[1], fields[0])
+
+            return b, a, fields
+
+        if self._isChildOf(tb, ta):
+            self.direct= True
+
+        return a, b, fields
+
+    def __len__(self):
+        return len(self._store)
+
+    def _selectWhat(self, fields, tbl= None):
+
+        if isinstance(fields, list):
+            if tbl: 
+                return _J(['%s.%s' % (tbl, f) for f in fields])
+            return _J(fields)
+        
+        if isinstance(fields, dict):
+            if tbl:
+                return _J(['%s.%s AS %s' % (tbl, k, v) 
+                    for k, v in fields.iteritems()])
+            return _J(['%s AS %s' % (k, v) 
+                for k, v in fields.iteritems()])
+
+        # recursion
+        sw= self._selectWhat
+
+        # no params:
+        if len(fields) == 2:
+            a= sw(fields[0], self.tblA)
+            b= sw(fields[1], self.tblB)
+
+            if a and b:
+                return _J([a, b])
+            return a or b
+
+        a= sw(fields[0], self.tblA)
+        b= sw(fields[1], self.tblB)
+        r= sw(fields[2], self.rtbl)
+
+        # if you provided three dict, it's possible
+        # to assume that params are specified.
+        if a and b:
+            return _J(a, b, r)
+
+        return _J(a or b, r)
+
+    def joinBuilders(self):
+
+	self.log_insane('MotherFusion: join relation')
+        # useful vars
+        ba= self.builderA
+        bb= self.builderB
+        ta= ba.table_name
+        tb= bb.table_name
+        r_tbl= self.rtbl
+
+        # Try to get relation table....
+        # If there is ambiguity, CRASH!
+        if r_tbl is None:
+            self.rtbl= r_tbl= self._searchRelation([ta, tb])
+
+        # Ok, the relation table is found
+        # Getting the mapping child dicts
+        a_d = self.getChildDeps(r_tbl, ba)
+        b_d = self.getChildDeps(r_tbl, bb)
+        
+        self.log_insane("Joining %s and %s with relation %s ",
+                         ta, tb, r_tbl)
+                        
+        # Make the base filter query ...
+        qry_filter = self._sqlFreeJoin(ta, r_tbl, tb, a_d, b_d)
+        ftr= self.filter
+        ftr.add_filter(qry_filter)
+        if self.jfilter:
+            ftr.add_filter(self.jfilter, self.rtbl)
+
+        params= self.params
+        if params:
+            if isinstance(params, list) or isinstance(params, dict):
+                self.fields = self.fields[0], self.fields[1], params
+            else:
+                # exclude redundant params...
+                params= [p for p in self._table_fields(r_tbl) 
+                        if p not in a_d.keys() + b_d.keys()]
+                # ... and add them
+                self.fields = self.fields[0], self.fields[1], params
+
+        what= self._selectWhat(self.fields)
+        distinct= self.distinct and 'DISTINCT' or ''
+        qry= 'SELECT %(distinct)s %(what)s FROM %(ta)s' % locals()
+
+        self._store= self.mr_query(qry, ftr)
+
+    def directJoin(self):
+
+        # useful vars
+        ba= self.builderA
+        bb= self.builderB
+        ta= ba.table_name
+        tb= bb.table_name
+	self.log_insane('MotherFusion: direct join: %s father, %s child', 
+			INF_COL(ta), INF_COL(tb))
+
+        what= self._selectWhat(self.fields)
+        jfilter, joining_table= self._sqlJoinParent(ba, bb)
+
+        distinct= self.distinct and 'DISTINCT' or ''
+        qry= 'SELECT %(distinct)s %(what)s FROM %(ta)s %(jfilter)s' % locals()
+        self._store= self.mr_query(qry, self.filter)
+
+    def getRecords(self, flag_obj= False):
+
+        if not flag_obj:
+            return self._store
+
+        fa= self._table_fields(self.tblA)
+        fb= self._table_fields(self.tblB)
+        ba= self.builderA
+        bb= self.builderB
+        s= self.session
+
+        dla= []
+        dlb= []
+        dlr= []
+        for rec in self._store:
+            da= {}
+            db= {}
+            dr= {}
+
+            for k, v in rec.iteritems():
+
+                if k in fa:
+                    da[k]= v
+                elif k in fb:
+                    db[k]= v
+                else:
+                    dr[k]= v
+
+                dla.append(da)
+                dlb.append(db)
+                if dr:
+                    dlr.append(dr)
+
+        rt= self.rtbl
+        if not rt or not dlr:
+            return [(ba(a, session= s), bb(b, session= s)) 
+                for a, b in zip(dla, dlb)]
+        return [(ba(a, session= s), getMotherObj(rt, rd, session= s), 
+                bb(b, session= s))
+            for a, r, b in zip(dla, dlr, dlb)]
+
+
